/*
 * Implements leacky bucket
 * $Id: leaky_bucket.c,v 1.2 1997/05/14 07:04:12 puhuri Exp $
 * $Log: leaky_bucket.c,v $
 * Revision 1.2  1997/05/14 07:04:12  puhuri
 * Ekojen mittausten versio.
 *
 * Revision 1.1  1996/11/27 10:42:47  puhuri
 * Initial revision
 *
 *
 */

#include <stdio.h>
#include "sched_tst.h"

struct bucket_state_vs {
  long long TAT;		/* Theoretical Arrival Time */
  long long I;			/* Increment */
  long long L;			/* Limit */
};

int GCRA_vs(long long ta, struct bucket_state_vs *bs)
{
  if (bs->TAT < ta) {
    bs->TAT = ta;
  } else {
    if (bs->TAT > ta + bs->L) {
#ifdef DEBUG
      printf ("TAT=%ld, ta = %ld, L = %ld, ta+L = %ld, d=%ld\n",
	      (long)bs->TAT, (long)ta, (long)bs->L, (long)(ta + bs->L),
	      (long)(bs->TAT - (ta + bs->L)));
#endif
      return 0;			/* non-conforming */
    }
  }

  bs->TAT += bs->I;
  return 1;			/* Conforming cell */
}

struct bucket_state_cs {
  int I;			/* Increment */
  int L;			/* Limit */
  int X;			/* Leaky Bucket counter */
  int LCT;			/* Last Copmliance time */
};

int GCRA_cs(int ta, struct bucket_state_cs *bs) {
  int Xp;			/* auxliary */

  Xp = bs->X - (ta - bs->LCT);
  if (Xp < 0) {
    Xp = 0;
  } else if (Xp > bs->L)
      return 0;			/* Non-Conforming */

  bs->X = Xp + 1;
  bs->LCT = ta;
  return 1;			/* Conforming cell */
}

int not_withn_contract (cell_log cells, tr_param tr)
{
  int i;
  struct bucket_state_vs bs_pcr, bs_scr;

  bs_pcr.I = tr.inv_pcr;	/* pcr = cells / s */
  bs_pcr.L = tr.cdvt;		/* cdvt = usec */
  bs_pcr.TAT = 0;

#ifdef DEBUG
  printf ("PCR: I=%ld L=%ld,\t", (long)bs_pcr.I, (long)bs_pcr.L);
#endif
  bs_scr.I = tr.inv_scr;	/* scr = cells / s */
  bs_scr.L = ((long long)tr.mbs - 1) * 
    ((long long)tr.inv_scr - (long long)tr.inv_pcr) + 
    (long long)tr.cdvt;
				/* mbs = cells */
  bs_scr.TAT = 0;

#ifdef DEBUG
  if (bs_scr.L < 1LL << 31)
    printf ("SCR: I=%ld L=%ld\n", (long)bs_scr.I, (long)bs_scr.L);
  else
    printf ("SCR: I=%ld L=%g\n", (long)bs_scr.I, (double)bs_scr.L);
#endif
  
  for (i = cells.min; i <= cells.max; i++) {
    if (GCRA_vs (cells.table[i], &bs_pcr)) {
      if (GCRA_vs (cells.table[i], &bs_scr)) {
      /* ok */
      } else {
	return -i;
      }
    } else {
      return i;			/* non-conforming cell */
    }
  }
  return 0;			/* tytt sopimuksen */
}
